#include "os.h"

/* Defines a GDT entry */
struct gdt_entry
{
    unsigned short limit_low;
    unsigned short base_low;
    unsigned char  base_middle;
    unsigned char  access;
    unsigned char  granularity;
    unsigned char  base_high;
}__attribute__((packed));

struct gdt_ptr
{
    unsigned short limit;
    unsigned int base;
}__attribute__((packed));

/* Our GDT, with 3 entries, and finally our special GDT pointer */
struct gdt_entry gdt[3];
struct gdt_ptr gp;

static void gdt_load(){ asm volatile("lgdt %0" : "=m" (gp)); }

/* Setup a descriptor in the Global Descriptor Table */
void gdt_set_gate(int num, unsigned long base, unsigned long limit, unsigned char access, unsigned char gran)
{
    /* Setup the descriptor base address */
    gdt[num].base_low = (base & 0xFFFF);
    gdt[num].base_middle = (base >> 16) & 0xFF;
    gdt[num].base_high = (base >> 24) & 0xFF;

    /* Setup the descriptor limits */
    gdt[num].limit_low = (limit & 0xFFFF);
    gdt[num].granularity = ((limit >> 16) & 0x0F);

    /* Finally, set up the granularity and access flags */
    gdt[num].granularity |= (gran & 0xF0);
    gdt[num].access = access;
}

/* Should be called by main. This will setup the special GDT pointer,
 * set up the first 3 entries in our GDT, and then finally call gdt_load() */
void gdt_install()
{
    /* Setup the GDT pointer and limit */
    gp.limit = (sizeof(struct gdt_entry) * 3);
    gp.base  = (unsigned int) &gdt;

    //TEST
    k_printf("GDT Base, GDT Size (byte), GDT Limit:", 17,0xA);

    char bufferGDTSIZE[20];
 	k_itoa(sizeof (struct gdt_entry),bufferGDTSIZE);
	char bufferGDTPBASE[20];
	k_itoa(gp.base,bufferGDTPBASE);
 	char bufferGDTPLIMIT[20];
	k_itoa(gp.limit,bufferGDTPLIMIT);

    k_printf(bufferGDTPBASE,  18,0xC);
    k_printf(bufferGDTSIZE,   19,0xA);
    k_printf(bufferGDTPLIMIT, 20,0xA);
    //TEST

    gdt_set_gate(0, 0, 0, 0, 0);                /* NULL descriptor */
    gdt_set_gate(1, 0, 0xFFFFFFFF, 0x9A, 0xCF); /* Code */
    gdt_set_gate(2, 0, 0xFFFFFFFF, 0x92, 0xCF); /* Data */

    gdt_load();
}
