#include "os.h"

// IDT entry
struct idt_entry
{
    unsigned short base_lo;
    unsigned short sel;
    unsigned char always0;
    unsigned char flags;
    unsigned short base_hi;
}__attribute__((packed)); //prevent compiler optimization

struct idt_ptr
{
    unsigned short limit;
    unsigned int base;
}__attribute__((packed)); //prevent compiler optimization

// Declare an IDT of 256 entries and a pointer to the IDT
struct idt_entry idt[256];
struct idt_ptr idtp;

static void idt_load(){ asm volatile("lidt %0" : "=m" (idtp)); } // load IDT register (IDTR)

// Put an entry into the IDT
void idt_set_gate(unsigned char num, unsigned long base, unsigned short sel, unsigned char flags)
{
    idt[num].base_lo = (base        & 0xFFFF);
    idt[num].base_hi = (base >> 16) & 0xFFFF;
    idt[num].sel     =   sel;
    idt[num].always0 =     0;
    idt[num].flags   = flags;
}

void idt_install()
{
    // Sets the special IDT pointer up
    idtp.limit = (sizeof (struct idt_entry) * 256);
    idtp.base  = (unsigned int) &idt;

    //TEST
    k_printf("IDT Base, IDT Size (byte), IDT Limit:", 21,0xA);

 	char bufferIDTSIZE[20];
 	k_itoa(sizeof (struct idt_entry),bufferIDTSIZE);
 	char bufferIDTPBASE[20];
	k_itoa(idtp.base,bufferIDTPBASE);
 	char bufferIDTPLIMIT[20];
	k_itoa(idtp.limit,bufferIDTPLIMIT);

    k_printf(bufferIDTPBASE,  22,0xC);
    k_printf(bufferIDTSIZE,   23,0xA);
    k_printf(bufferIDTPLIMIT, 24,0xA);
    //TEST

    // Clear out the entire IDT, initializing it to zeros
    k_memset(&idt, 0, sizeof(struct idt_entry) * 256);

    // Add any new ISRs to the IDT here using idt_set_gate

    // Points the IDTR to the IDT
    idt_load();
}
