/* PrettyOS is the result of a stepwise OS development
 * shown at www.henkessoft.de */

#ifndef OS_H
#define OS_H

#define TRUE		1
#define FALSE		0

typedef unsigned int size_t;
typedef unsigned int   UINT;
typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned long  ULONG;

/* This defines what the stack looks like after an ISR was running */
struct regs
{
    unsigned int gs, fs, es, ds;
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;
    unsigned int int_no, err_code;
    unsigned int eip, cs, eflags, useresp, ss;
};

extern void k_clear_screen();
extern unsigned int k_printf(char* message, unsigned int line, char attribute);
extern void update_cursor(int row, int col);

extern void keyboard_init();
extern unsigned char const k_getch();
extern unsigned int FetchScancode();
extern unsigned int FetchAndAnalyzeScancode();
extern void keyboard_handler(struct regs* r);

extern void outportb(unsigned port, unsigned val);
inline unsigned inportb(unsigned port);
extern void* k_memset(void *dest, char val, size_t count);
extern unsigned short* k_memsetw(unsigned short *dest, unsigned short val, size_t count);
extern void* k_memcpy(void* dest, const void* src, size_t count);
extern size_t k_strlen(const char *str);
extern void reboot();

extern void cli();
extern void sti();
extern void k_itoa(int value, char* valuestring);
extern void k_i2hex(unsigned int val, unsigned char* dest, int len);
extern void float2string(float value, int decimal, char* valuestring);

extern void gdt_set_gate(int num, unsigned long base, unsigned long limit, unsigned char access, unsigned char gran);
extern void gdt_install();

extern void idt_set_gate(unsigned char num, unsigned long base, unsigned short sel, unsigned char flags);
extern void idt_install();

extern void isrs_install();
extern void irq_install();
extern void timer_install();
extern void keyboard_install();
extern void fault_handler(struct regs* r);
extern void irq_install_handler(int irq, void (*handler)(struct regs* r));
extern void irq_uninstall_handler(int irq);
extern void irq_remap(void);
extern void irq_handler(struct regs* r);

extern int k_power(int base,int n);



#endif
